# %Z%%M% %I% %W% %G% %U% 
# ===========================================================================
# IBM Confidential
# OCO Source Materials
# Licensed Materials - Property of IBM
#
# 5724-S24
#
# (C) Copyright IBM Corp. 2010 , 2011
#  All Rights Reserved.
#
# The source code for this program is not published or otherwise divested of
# its trade secrets, irrespective of what has been deposited with the U.S.
# Copyright Office.
# ===========================================================================
#
#  wsadmin -lang jython  -f RTSSWSSecurity.py  --a configure --n [pdtname] [--t clustername] {--k [rtssjks-pw] | --p [rtssjks-pw-file]} --s [sourcedir] 
#  wsadmin -lang jython  -f RTSSWSSecurity.py  --a unconfigure --n [pdtname] [--t clustername] {--k [rtssjks-pw] | --p [rtssjks-pw-file]} --s [sourcedir] 
#  wsadmin -lang jython  -f RTSSWSSecurity.py  --a reconfigure --n [pdtname] [--t clustername] {--k [rtssjks-pw] | --p [rtssjks-pw-file]} --s [sourcedir]
#  wsadmin -lang jython  -f RTSSWSSecurity.py  --a show [--t clustername] 
#  wsadmin -lang jython  -f RTSSWSSecurity.py  --h
# 
#   Windows usage tip:  Use forward slashes in directory paths
#
import getopt, sys
import os
import java.lang.System as System


lineSeparator = java.lang.System.getProperty('line.separator')

# Application specific settings

PDT_NAME = ""

RTSS = "IBM Tivoli Runtime Security Services"
RTSS_AGENT = "IBM Tivoli Runtime Security Services Agent"

SUFFIX = "OU=tivoli, O=ibm, C=us"
CN = PDT_NAME
DN = 'CN='+CN+', '+SUFFIX

KEYSTORENAME = "RTSS"
KEYSTOREPASSWORD = ""
KEYNAME = DN
KEYALIAS = PDT_NAME
TRUSTSTORENAME = "RTSS"
TRUSTSTOREPASSWORD = ""
TRUSTANCHORNAME = "RTSSTrustAnchor"
PASSWORDFILE = ""

APPLICATION = RTSS

SOURCEDIR = ""
ACTION = "show"

True = 1
False = 0

DEBUG = False
TRACE = False

if (DEBUG == True):
	TRACE = True
#fi

#
# Get basic info, cell node server
#   We have to calculate nodename or clustername depending on the deployment
#

VERSION = 6
CELL = AdminConfig.list("Cell" )
CELLNAME = AdminConfig.showAttribute(AdminConfig.list("Cell" ), "name" )
NODENAME = ""
CLUSTERNAME = ""
SCOPE = ""
RTSS_JKS='${CONFIG_ROOT}/cells/'+CELLNAME+'/rtss.jks'

class ConfigurationException:
	def __init__(self, message):
		self._message = message
		
		print message
	#endif
#endclass

class WSSecurity:

	def __init__(self):
		global APPLICATION
		self._applicationName = APPLICATION
		self._attachmentId = 0
		self._resourceName = ""
		self._webServiceResource = ""
		self._policySet = ""
		self._binding = ""
		self._policySetDirectory = ""
	#enddef


	#
	# Setters
	#
	def setResourceName(self, resourceName):
		self._resourceName = resourceName
	#enddef

	def setPolicySet(self, policySet):
		self._policySet = policySet
	#enddef

	def setBinding(self, binding):
		self._binding = binding
	#enddef

	def setPolicySetDirectory(self, policySetDirectory):
		self._policySetDirectory = policySetDirectory
	#enddef

	def setManagementScope(self, scope):
		self._scope = scope
	#enddef


	#
	# Utility methods
	#
	def getAttribute(self, nvListString, key):
		
		result = ""
		#print nvListString
		
		nvList = nvListString.replace("] [","], [")
		nvList = nvList.replace("]","")
		nvList = nvList.replace("[","")
		#print nvList
		
		pairs = nvList.split(',')
		for pair in pairs:
			nvPair=pair.split()
			name=nvPair[0]
			value=nvPair[1]
			#print "name:  " + name
			#print "value: " + value 
			if (name == key):
				result = value
				break
			#endif
		#endfor
		
	
		return result
	#endif

	def queryAttachmentID(self, applicationName, resourceName):
		attachmentID = 0

		attachmentsList = AdminTask.getPolicySetAttachments('[-applicationName "' + applicationName +   '" -attachmentType application -expandResources '+ resourceName +']')
		if len(attachmentsList) > 0 :
			for attachment in attachmentsList.split(lineSeparator):
				# print attachment
				directAttachment=self.getAttribute(attachment, "directAttachment")
				if (directAttachment == "true"):
					attachmentID=self.getAttribute(attachment, "attachmentId")
					break
				#endif
				#print attachmentID
			#endfor
		#endif
		return attachmentID
	#enddef
	

	def saveAll(self):

		print "Saving configuration. This may take a few minutes."
		AdminConfig.save()
		print "Configuration saved."

	#enddef


	#
	# Policy Sets
	#
	def importPolicySet(self, sourceDirectory, policySet):

		policySetFile = sourceDirectory + '/' + policySet + '.zip'
		print "Importing policy set = %s"  %(policySetFile)
		AdminTask.importPolicySet('[-importFile ' + policySetFile + ' ]') 

	#enddef



	def deletePolicySet(self, policySetName):
		
		try:
			ps = AdminTask.getPolicySet('[-policySet "'+policySetName+'"]')
		except:
			## we'll get an exception if the policy set doesn't exist
			ps = ""
	 	
		if (len(ps)>0):
			print "Removing %s " %(policySetName)
			AdminTask.deletePolicySet('[-policySet '+policySetName+']')
		#endif 
			
	#enddef


	def attachPolicySet(self, applicationName, policySet, webresource):
		attachmentID = 0
		try:
			attachPolicySetCommand = '[-applicationName "'+ applicationName +'" -policySet '+policySet+' -resources '+webresource+' -attachmentType application]'
			attachmentID = AdminTask.createPolicySetAttachment(attachPolicySetCommand)
			print "Created policy attachment ID %d" %(attachmentID)
		except:
			e = sys.exc_info()[1]
			print e
			print "Failed attach policy set command: %s" %(attachPolicySetCommand)
			raise e
		#endtry
		
		return attachmentID
	#enddef


	def detachPolicySet(self, applicationName, attachmentID):

		print "Removing policy set attachment for %s" %(attachmentID)
		if (attachmentID != 0 ):
			AdminTask.deletePolicySetAttachment('[-applicationName "' + applicationName + '" -attachmentId ' +attachmentID + ' -attachmentType application]')
		#endif
		
	#enddef

	#
	# Bindings
	#
	def createBinding(self, applicationName, attachmentID, bindingName):
		print "Creating bindings for attachmentID %s" %(attachmentID)
		AdminTask.setBinding('[-policyType WSSecurity   -bindingLocation "[ [application ['+applicationName+']] [attachmentId '+attachmentID+'] ]" -attributes "[application.name application]" -attachmentType application -bindingName '+bindingName+']')
		AdminTask.setBinding('[-policyType WSAddressing -bindingLocation "[ [application ['+applicationName+']] [attachmentId '+attachmentID+'] ]" -attributes -attachmentType application -bindingName '+bindingName+']')
	#enddef
	
	def removeBindingTypes(self, bindingTypesList, applicationName, attachmentID, bindingName):
		
		for bindingType in bindingTypesList:
			try:
				AdminTask.setBinding('[-policyType '+bindingType+' -bindingLocation "[ [application ['+applicationName+']] [attachmentId '+attachmentID+'] ]" -attachmentType application -remove true]')
			except:
				pass
			#endTry
		#endfor 

	#enddef
	
	def queryBindingTypes(self, applicationName, attachmentID):
		
		policyTypes = AdminTask.listPolicyTypes('[-bindingLocation "[ [application ['+applicationName+']] [attachmentId '+attachmentID+'] ]" -attachmentType application]')
		return policyTypes
	#endif 
	
	
	def setBinding(self, applicationName, attachmentID, attributes):
		global TRACE
		global True
		global KEYSTOREPASSWORD

		try:
			bindingLocation = '"[ [application ['+applicationName+']] [attachmentId '+attachmentID+'] ]"'		
			bindingCommand = '[-policyType WSSecurity -bindingLocation '+bindingLocation+' -attributes "[ '+attributes+']" -attachmentType application]'
			if (TRACE == True):
				bindingCommandTrace = bindingCommand
				if len(KEYSTOREPASSWORD) != 0:
					bindingCommandTrace = bindingCommand.replace("keypass " + KEYSTOREPASSWORD,"keypass ****")
				#endif
				print 'AdminTask.setBinding(%s)' %(bindingCommandTrace)
			#endif
			AdminTask.setBinding(bindingCommand)
		except:
			e = sys.exc_info()[1]
			print e
			if len(KEYSTOREPASSWORD) != 0:
				bindingCommandTrace = bindingCommand.replace("keypass " + KEYSTOREPASSWORD,"keypass ****")
			#endif
			print "Failed binding command: %s" %(bindingCommandTrace)
			raise e
		#end try
	#enddef
	
	
	#
	# Top level configure binding method
	def configureBinding(self):
		
		self.configureTrustStoreAnchor()
		self.configureAuthenticationToken()
		self.configureAsymmetricBindingTokens()
		self.configureRequestProtection()
		self.configureResponseProtection()
		self.configureAuthenticationCaller()
		
	#enddef
	
	def unconfigureBinding(self):
		pass
	#endif

	def configureAsymmetricBindingTokens(self):

		self.configureAsymmetricBindingRecipientEncryptionToken()  # pdt + reg
		self.configureAsymmetricBindingRecipientSignatureToken()  # pdt + reg

		self.configureAsymmetricBindingInitiatorEncryptionToken()  # reg only
		self.configureAsymmetricBindingInitiatorSignatureToken()   # reg only
	#enddef
	
	def configureRequestProtection(self):	
		self.configureRequestEncryptionProtection()
		self.configureRequestSignatureProtection()
	#enddef

	def configureResponseProtection(self):	
		self.configureResponseEncryptionProtection()
		self.configureResponseSignatureProtection()
	#enddef
	
	#this method intended to be overridden
	def configureAuthenticationCaller(self):
		pass
	#enddef

	#this method intended to be overridden
	def configureTrustStoreAnchor(self):
		pass
	#enddef
	
	# this method intended to be overridden
	def configureAuthenticationToken(self):
		pass
	#enddef

	# this method intended to be overridden
	def configureAsymmetricBindingInitiatorEncryptionToken(self):
		pass
	#enddef
	# this method intended to be overridden
	def configureAsymmetricBindingInitiatorSignatureToken(self):
		pass
	#enddef
	# this method intended to be overridden
	def configureAsymmetricBindingRecipientEncryptionToken(self):
		pass
	#enddef
	# this method intended to be overridden
	def configureAsymmetricBindingRecipientSignatureToken(self):
		pass
	#enddef
	
	# this method intended to be overridden
	# request:app_encparts
	def configureRequestEncryptionProtection(self):
		pass
	#enddef

	# this method intended to be overridden
	# request:app_signparts
	def configureRequestSignatureProtection(self):
		pass
	#enddef

	# this method intended to be overridden
	#  response:app_encparts
	def configureResponseEncryptionProtection(self):
		pass
	#enddef

	# this method intended to be overridden
	#     response:app_signparts
	def configureResponseSignatureProtection(self):
		pass
	#enddef
	
	def configureTrustStoreAnchorBinding(self, applicationName, attachmentID, trustStoreName, trustAnchorName, scope):

		attributes = ' '\
		'[application.securityinboundbindingconfig.trustanchor_999.keystore.keystoreref [name='+trustStoreName+' managementScope=' +scope+ ']] '\
		'[application.securityinboundbindingconfig.trustanchor_999.name '+ trustAnchorName +'] '
		
		self.setBinding(applicationName, attachmentID, attributes)

	#enddef

	# Username token authentication
	def configureUserNameTokenBinding(self, requestTokenName, applicationName, attachmentID, trustStoreName, trustAnchorName):
		
		name = requestTokenName
		name = "token_auth"
		
		# confgure UNT
		attributes = ''\
		'[application.securityinboundbindingconfig.tokenconsumer_999.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#UsernameToken] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.jaasconfig.configname system.wss.consume.unt] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.name request:'+name+'] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.securitytokenreference.reference request:'+name+'] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.UNTConsumeCallbackHandler]'

		self.setBinding(applicationName, attachmentID, attributes);

		##configure UNT callback handler
		attributes = ''\
		'[application.name application] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.UNTConsumeCallbackHandler] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#UsernameToken] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.jaasconfig.configname system.wss.consume.unt] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.name request:'+name+'] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.securitytokenreference.reference request:'+name+'] '
		
		self.setBinding(applicationName, attachmentID, attributes)

	#enddef

	# X509 authentication
	# request:token-auth
	def configureX509TokenBinding(self, applicationName, attachmentID, trustStoreName, trustAnchorName):
		
		# confgure X509 Token
		attributes = ''\
		'[application.securityinboundbindingconfig.tokenconsumer_999.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.jaasconfig.configname system.wss.consume.x509] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.name request:token-auth] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.securitytokenreference.reference request:token-auth] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509ConsumeCallbackHandler] '

		self.setBinding(applicationName, attachmentID, attributes);

		
		##configure X509 callback handler
		attributes = ''\
		'[application.name application] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.securitytokenreference.reference request:token-auth] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.name request:token-auth] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509ConsumeCallbackHandler] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.jaasconfig.configname system.wss.consume.x509] '\
		'[application.securityinboundbindingconfig.tokenconsumer_0.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '\

		self.setBinding(applicationName, attachmentID, attributes)
	#enddef

		
	#
	# Public-ish methods
	#

	def unconfigure(self):

		self._attachmentId = self.queryAttachmentID(self._applicationName, self._resourceName)

		if (self._attachmentId != 0):
			bindingTypes = self.queryBindingTypes(self._applicationName, self._attachmentId)
			if (len(bindingTypes)>0):
				self.removeBindingTypes(bindingTypes.split(), self._applicationName, self._attachmentId, self._binding)
			#endif
		#endif
			
		if (self._attachmentId != 0):
	
			self.detachPolicySet(self._applicationName, self._attachmentId)
		#endif

		self.deletePolicySet(self._policySet)

	#enddef



	def configure(self):

		self.importPolicySet(self._policySetDirectory, self._policySet)
	
		self.attachPolicySet(self._applicationName, self._policySet, self._webServiceResource)
        
		self._attachmentId = self.queryAttachmentID(self._applicationName, self._resourceName)
        
		self.createBinding(self._applicationName, self._attachmentId, self._binding)
		
		self.configureBinding()

	#enddef


	def show(self):
		print self
		print self._applicationName 
		print 'Policy set name = %s' %(self._policySet)
		print 'Policy set binding name = %s' %(self._binding)
		attachmentId = self.queryAttachmentID(self._applicationName, self._resourceName)
		print 'Attachment ID: %d' %(attachmentId)
	#enddef


class RTSSAdminService(WSSecurity):

	RTSSADMINSERVICE_ATTACHMENT_ID = 0
	#RTSSADMINSERVICE_RESOURCE_NAME = "{http://command.admin.rtss.tscc.ibm.com/servicesAdminCommand}servicesAdminCommand"
	RTSSADMINSERVICE_RESOURCE_NAME = "{http://command.admin.rtss.tscc.ibm.com}servicesAdminCommand"
	RTSSADMINSERVICE_POLICYSET = "RTSSAdminCommandServicePolicySet"
	RTSSADMINSERVICE_BINDING = "RTSSAdminCommandServiceBinding"
	RTSSADMINSERVICE_WEB_RESOURCE_NAME = "WebService:/com.ibm.tscc.rtss.admin.ws.war:{http://command.admin.rtss.tscc.ibm.com}servicesAdminCommand"
	                                       
	
	
	def __init__(self):
		global APPLICATION
		global CELLNAME
		global NODENAME
		global KEYSTORENAME
		global KEYSTOREPASSWORD		
		global TRUSTSTORENAME
		global TRUSTSTOREPASSWORD
		global TRUSTANCHORNAME 
		global SOURCEDIR
		
		global PDT_NAME
		global KEYALIAS
		global KEYNAME

		## vars needed for policyset & binding creation
		self._applicationName = APPLICATION
		self._attachmentId = 0
		self._resourceName = self.RTSSADMINSERVICE_RESOURCE_NAME
		self._webServiceResource = self.RTSSADMINSERVICE_WEB_RESOURCE_NAME
		self._policySet = self.RTSSADMINSERVICE_POLICYSET
		self._binding = self.RTSSADMINSERVICE_BINDING
		self._policySetDirectory = SOURCEDIR
		
		## vars needed for binding configuration
		self._keyStoreName = KEYSTORENAME
		self._keyStorePassword = KEYSTOREPASSWORD
		self._trustStoreName = TRUSTSTORENAME
		self._trustStorePassword = TRUSTSTOREPASSWORD
		self._trustAnchorName = TRUSTANCHORNAME
		self._cellName = CELLNAME
		self._scope = SCOPE

		self._pdtName = PDT_NAME # "kaa-rtss"
		self._keyStoreAlias = KEYALIAS 
		self._trustStoreAlias = KEYALIAS 
		self._keyName = KEYNAME # 'CN='+CN+', '+SUFFIX
		
	#enddef
	
	def hasRTSSKeyStore(self, scope):
		global KEYSTORENAME # "RTSS"
		hasKeyStore = False
		
		keystoreList = AdminTask.listKeyStores('[-scopeName '+scope+' ]')
		keystores = keystoreList.split()
		for keystore in keystores:
			#print keystore
			if keystore.startswith(KEYSTORENAME):
				hasKeyStore = True
				break
			#endif
		#endfor
		
		return hasKeyStore
	#enddef
		

	def createRTSSKStore(self, scope):
		global KEYSTORENAME # "RTSS"
		global KEYSTOREPASSWORD # "secret"
		global KEYNAME # DN
		global KEYALIAS # PDT_NAME
		global TRUSTSTORENAME # "RTSS"
		global TRUSTSTOREPASSWORD # "secret"
		global TRUSTANCHORNAME # "RSSMTrustAnchor"

		hasKeyStore = self.hasRTSSKeyStore(scope)
		if (hasKeyStore == False):
			print "Creating %s key store" %(KEYSTORENAME)
			command = '[-keyStoreName  '+KEYSTORENAME+' -keyStoreType JKS -keyStoreLocation '+RTSS_JKS+' '\
			'-keyStorePassword '+KEYSTOREPASSWORD+' -keyStorePasswordVerify '+KEYSTOREPASSWORD+' '\
			'-scopeName '+scope+' ]'
			AdminTask.createKeyStore(command)
		#endif

	#enddef


	def deleteRTSSKStore(self, scope):
		global True
		global KEYSTORENAME # "RTSS"
		global KEYSTOREPASSWORD # "secret"
		global KEYNAME # DN
		global KEYALIAS # PDT_NAME
		global TRUSTSTORENAME # "RTSS"
		global TRUSTSTOREPASSWORD # "secret"
		global TRUSTANCHORNAME # "RSSMTrustAnchor"

		#hasKeyStore = False
		#
		#keystoreList = AdminTask.listKeyStores('[-scopeName (cell):'+self._cellName+':(node):'+self._nodeName+' ]')
		#keystores = keystoreList.split()
		#for keystore in keystores:
		#	print keystore
		#	if keystore.startswith(KEYSTORENAME):
		#		hasKeyStore = True
		#		break
		#	#endif
		##endfor
		
		hasKeyStore = self.hasRTSSKeyStore(scope)
		
		if (hasKeyStore == True):
			print "Deleting %s key store" %(KEYSTORENAME)
			command = ('[-keyStoreName '+KEYSTORENAME+' -scopeName '+scope+' ]')
			AdminTask.deleteKeyStore(command)
		#endif


	#enddef
	
	# 
	# 
	# 
	def configureTrustStoreAnchor(self):
		self.configureTrustStoreAnchorBinding(self._applicationName, self._attachmentId, self._trustStoreName, self._trustAnchorName, self._scope)
	#enddef
	
	
	# 
	# 
	# use X509Cert
	def configureAuthenticationToken(self):
		# skip the explicit auth token .. use the signature token instead
		
		# Configure X509 Authentication
		#self.configureX509TokenBinding(self._applicationName, self._attachmentId, self._trustStoreName, self._trustAnchorName)
		pass
	#enddef


	# 
	# 
	# 
	def configureAsymmetricBindingRecipientEncryptionToken(self):
		
		#token		
		# http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3
		# http://docs.oasis-open.org/wss/v1.1/wss-v1.1-spec-pr-x509TokenProfile-01#X509v3
		attributes = '' \
		'[application.securityinboundbindingconfig.tokenconsumer_999.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] ' \
		'[application.securityinboundbindingconfig.tokenconsumer_999.jaasconfig.configname system.wss.consume.x509]  ' \
		'[application.securityinboundbindingconfig.tokenconsumer_999.name AsymmetricBindingRecipientEncryptionToken0] ' \
		'[application.securityinboundbindingconfig.tokenconsumer_999.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] ' \
		'[application.securityinboundbindingconfig.tokenconsumer_999.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509ConsumeCallbackHandler] ' 
		
		#self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		# callback handler
		attributes = '' \
		'[application.name application] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.jaasconfig.configname system.wss.consume.x509] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.callbackhandler.key.name ['+self._keyName+']] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509ConsumeCallbackHandler] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.callbackhandler.keystore.keystoreref [name='+self._keyStoreName+' managementScope='+self._scope+']] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.callbackhandler.key.keypass '+self._keyStorePassword+'] ' \
		'[application.securityinboundbindingconfig.tokenconsumer_1.callbackhandler.certpathsettings.trustanchorref.reference '+self._trustAnchorName+'] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.callbackhandler.key.alias '+self._keyStoreAlias+'] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.name AsymmetricBindingRecipientEncryptionToken0] '\
		'[application.securityinboundbindingconfig.tokenconsumer_1.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] '\
		
		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		
	#enddef

	# 
	# 
	# 
	def configureAsymmetricBindingRecipientSignatureToken(self):

		# token		 
		# callback handler
		attributes = '' \
		'[application.securityoutboundbindingconfig.tokengenerator_999.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509GenerateCallbackHandler] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.callbackhandler.key.alias '+self._keyStoreAlias+'] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.callbackhandler.key.keypass '+self._keyStorePassword+'] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.callbackhandler.key.name ['+self._keyName+']] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.callbackhandler.keystore.keystoreref [name='+self._keyStoreName+' managementScope='+self._scope+']] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenGenerator] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.jaasconfig.configname system.wss.generate.x509] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.name AsymmetricBindingRecipientSignatureToken0] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '
		
		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		# callback handler
		#attributes = '' \
		#'[application.name application] '\
		#'[application.securityinboundbindingconfig.trustanchor_0.keystore.keystoreref [name='+self._trustStoreName+' managementScope=(cell):'+self._cellName+':(node):'+self._nodeName+']] '\
		#'[application.securityinboundbindingconfig.trustanchor_0.name '+self._trustStoreName+'] '
		
		#self.setBinding(self._applicationName, self._attachmentId, attributes)
		

	#enddef
	
	# 
	# 
	# request:app_encparts
	def configureRequestEncryptionProtection(self):
		
		# create encrypt key info
		attributes = ''\
		'[application.securityinboundbindingconfig.keyinfo_999.name encrypt-message-key]' \
		'[application.securityinboundbindingconfig.keyinfo_999.tokenreference.reference AsymmetricBindingRecipientEncryptionToken0]'\
		'[application.securityinboundbindingconfig.keyinfo_999.classname com.ibm.ws.wssecurity.wssapi.CommonContentConsumer]' 

		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		#encrypt part
		attributes = ''\
		'[application.securityinboundbindingconfig.encryptioninfo_900.name encrypt-message-part] '\
		'[application.securityinboundbindingconfig.encryptioninfo_900.keyencryptionkeyinfo_0.reference encrypt-message-key] '\
		'[application.securityinboundbindingconfig.encryptioninfo_900.encryptionpartreference.reference request:app_encparts] '
		
		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
	#enddef

	# 
	# 
	#     response:app_signparts
	def configureResponseSignatureProtection(self):
		
		# sign message key
		attributes = ''\
		'[application.securityoutboundbindingconfig.keyinfo_0.name signature-message-key] '\
		'[application.securityoutboundbindingconfig.keyinfo_0.tokenreference.reference AsymmetricBindingRecipientSignatureToken0] '\
		'[application.securityoutboundbindingconfig.keyinfo_0.type EMB]'\
		'[application.securityoutboundbindingconfig.keyinfo_0.classname com.ibm.ws.wssecurity.wssapi.CommonContentGenerator]'
		 
		self.setBinding(self._applicationName, self._attachmentId, attributes)

		# sign message part
		attributes = ''\
		'[application.securityoutboundbindingconfig.signinginfo_900.name signature-message-part] '\
		'[application.securityoutboundbindingconfig.signinginfo_900.signingpartreference_900.reference response:app_signparts] '\
		'[application.securityoutboundbindingconfig.signinginfo_900.order 1] '\
		'[application.securityoutboundbindingconfig.signinginfo_900.signingkeyinfo_0.reference signature-message-key]'
		 
		self.setBinding(self._applicationName, self._attachmentId, attributes)

		# transform alg
		attributes = ''\
		'[application.securityoutboundbindingconfig.signinginfo_0.signingpartreference_0.transform_999.algorithm http://www.w3.org/2001/10/xml-exc-c14n#]'
	
		self.setBinding(self._applicationName, self._attachmentId, attributes)
	#enddef


	#
	#
	#
	def configureAsymmetricBindingInitiatorEncryptionToken(self):

		# Set token
		attributes = '' \
		'[application.securityoutboundbindingconfig.tokengenerator_999.name AsymmetricBindingInitiatorEncryptionToken0] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenGenerator] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509GenerateCallbackHandler] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.jaasconfig.configname system.wss.generate.x509] '\
		'[application.securityoutboundbindingconfig.tokengenerator_999.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '

		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		# Set the callback handler
		attributes = '' \
		'[application.securityoutboundbindingconfig.tokengenerator_1.callbackhandler.properties_0.name com.ibm.wsspi.wssecurity.token.cert.useRequestorCert] '\
		'[application.securityoutboundbindingconfig.tokengenerator_1.callbackhandler.key.alias '+self._keyStoreAlias+'] '\
		'[application.securityoutboundbindingconfig.tokengenerator_1.callbackhandler.key.name ['+self._keyName+']] '\
		'[application.securityoutboundbindingconfig.tokengenerator_1.callbackhandler.keystore.keystoreref [name='+self._trustStoreName+' managementScope=' +self._scope+']] '\
		'[application.securityoutboundbindingconfig.tokengenerator_1.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509GenerateCallbackHandler] '\
		'[application.securityoutboundbindingconfig.tokengenerator_1.callbackhandler.properties_0.value true] '
	
		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
	#enddef


	# 
	# 
	#
	def configureAsymmetricBindingInitiatorSignatureToken(self):
		# Set token
		attributes = '' \
		'[application.securityinboundbindingconfig.tokenconsumer_999.name AsymmetricBindingInitiatorSignatureToken0] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.jaasconfig.configname system.wss.consume.x509] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] '\
		'[application.securityinboundbindingconfig.tokenconsumer_999.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509ConsumeCallbackHandler] '

		#self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		# Set the callback handler
		# Include the magic com.ibm.ws.console.webservices.policyset.protectionToken property
		# need to add
		#  <keyStore KeyStoreRef="name=RTSS managementScope=(cell):bagheeraNode05Cell:(node):bagheeraNode05"/>
                # <key name="CN=bagheera-rtssclient-remote, OU=tivoli, O=ibm, C=us" alias="bagheera-rtssclient-remote"/>
		attributes = '' \
		'[application.securityinboundbindingconfig.tokenconsumer_2.callbackhandler.certpathsettings.trustanchorref.reference '+self._trustAnchorName+'] '\
		'[application.securityinboundbindingconfig.tokenconsumer_2.callbackhandler.classname com.ibm.websphere.wssecurity.callbackhandler.X509ConsumeCallbackHandler] '\
		'[application.securityinboundbindingconfig.tokenconsumer_2.classname com.ibm.ws.wssecurity.wssapi.token.impl.CommonTokenConsumer] '\
		'[application.securityinboundbindingconfig.tokenconsumer_2.callbackhandler.properties_0.name com.ibm.ws.console.webservices.policyset.protectionToken]'\
		'[application.securityinboundbindingconfig.tokenconsumer_2.callbackhandler.properties_0.value true] '\
		'[application.securityinboundbindingconfig.tokenconsumer_2.jaasconfig.configname system.wss.consume.x509] '\
		'[application.securityinboundbindingconfig.tokenconsumer_2.name AsymmetricBindingInitiatorSignatureToken0] '\
		'[application.securityinboundbindingconfig.tokenconsumer_2.valuetype.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '

		self.setBinding(self._applicationName, self._attachmentId, attributes)
	#enddef

	

	# 
	#
	# request:app_signparts
	def configureRequestSignatureProtection(self):
		# inbound sign key
		attributes = '' \
		'[application.securityinboundbindingconfig.keyinfo_999.name inbound-signature-message-key] '\
		'[application.securityinboundbindingconfig.keyinfo_999.tokenreference.reference AsymmetricBindingInitiatorSignatureToken0] '\
		'[application.securityinboundbindingconfig.keyinfo_999.classname com.ibm.ws.wssecurity.wssapi.CommonContentConsumer] '

		
		self.setBinding(self._applicationName, self._attachmentId, attributes)

		# callback handler
		attributes = '' \
		'[application.securityinboundbindingconfig.signinginfo_901.name signature-message-part] '\
		'[application.securityinboundbindingconfig.signinginfo_901.signingpartreference_900.reference request:app_signparts] '\
		'[application.securityinboundbindingconfig.signinginfo_901.signingkeyinfo_0.reference inbound-signature-message-key] '\
		'[application.securityinboundbindingconfig.signinginfo_901.properties_0.value true] '\
		'[application.securityinboundbindingconfig.signinginfo_901.properties_0.name com.ibm.ws.console.webservices.policyset.protectionToken] '
		
		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		# transform alg
		# AdminTask.setBinding('[-policyType WSSecurity -bindingLocation "[ [application [Tivoli Runtime Security Services Agent]] [attachmentId 341] ]" -attributes "[application.securityinboundbindingconfig.signinginfo_0.signingpartreference_0.transform_999.algorithm http://www.w3.org/2001/10/xml-exc-c14n#]" -attachmentType application]') 
		attributes = ''\
		'[application.securityinboundbindingconfig.signinginfo_901.signingpartreference_0.transform_999.algorithm http://www.w3.org/2001/10/xml-exc-c14n#]'

		attributes = ''\
		'[application.securityinboundbindingconfig.signinginfo_0.signingpartreference_0.transform_999.algorithm http://www.w3.org/2001/10/xml-exc-c14n#]' 
	
		self.setBinding(self._applicationName, self._attachmentId, attributes)
	#enddef		


	#  response:app_encparts
	def configureResponseEncryptionProtection(self):
		# outbound-encryption-key
		attributes = '' \
		'[application.securityoutboundbindingconfig.keyinfo_999.name outbound-encryption-key] '\
		'[application.securityoutboundbindingconfig.keyinfo_999.tokenreference.reference AsymmetricBindingInitiatorEncryptionToken0] '\
		'[application.securityoutboundbindingconfig.keyinfo_999.type KEYID] '\
		'[application.securityoutboundbindingconfig.keyinfo_999.classname com.ibm.ws.wssecurity.wssapi.CommonContentGenerator] '
		
		self.setBinding(self._applicationName, self._attachmentId, attributes)

		attributes = ''\
		'[application.securityoutboundbindingconfig.keyinfo_900.name outbound-encryption-key] '\
		'[application.securityoutboundbindingconfig.keyinfo_900.tokenreference.reference AsymmetricBindingInitiatorEncryptionToken0] '\
		'[application.securityoutboundbindingconfig.keyinfo_900.type KEYID] '\
		'[application.securityoutboundbindingconfig.keyinfo_900.classname com.ibm.ws.wssecurity.wssapi.CommonContentGenerator] '
		self.setBinding(self._applicationName, self._attachmentId, attributes)
		
		# callback handler.. ignore the log message
		attributes = '' \
		'[application.securityoutboundbindingconfig.encryptioninfo_900.name outbound-encryption-part] '\
		'[application.securityoutboundbindingconfig.encryptioninfo_900.encryptionpartreference.reference response:app_encparts] '\
		'[application.securityoutboundbindingconfig.encryptioninfo_900.keyencryptionkeyinfo_0.reference outbound-encryption-key] '\
		'[application.securityoutboundbindingconfig.encryptioninfo_900.order 2] '

		self.setBinding(self._applicationName, self._attachmentId, attributes)
	#enddef
	
		
	#
	# Note that app_signparts needs to line up with configureResponseSignatureProtection
	def configureAuthenticationCaller(self):
		
		global VERSION
		
		# WAS 7 (and higher?) needs the order set.   WAS 6 will reject an order
		if (VERSION.startswith("6")):
			attributes = ''\
			'[application.securityinboundbindingconfig.caller_0.calleridentity.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '\
			'[application.name application] '\
			'[application.securityinboundbindingconfig.caller_0.jaasconfig.configname system.wss.caller] '\
			'[application.securityinboundbindingconfig.caller_0.name x509-authn-caller] '\
			'[application.securityinboundbindingconfig.caller_0.requiredsigningpartreference.reference request:app_signparts] '
		else:
			attributes = ''\
			'[application.securityinboundbindingconfig.caller_0.calleridentity.localname http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3] '\
			'[application.name application] '\
			'[application.securityinboundbindingconfig.caller_0.jaasconfig.configname system.wss.caller] '\
			'[application.securityinboundbindingconfig.caller_0.name x509-authn-caller] '\
			'[application.securityinboundbindingconfig.caller_0.requiredsigningpartreference.reference request:app_signparts] '\
			'[application.securityinboundbindingconfig.caller_0.order 1] '
		
		#endif
			
		self.setBinding(self._applicationName, self._attachmentId, attributes)
	#enddef

#endclass RTSSAdminService

#
#     Main 
#


def unconfigureServices(scope, removeJKS, doSave):
	global True

	service = RTSSAdminService()
	service.setManagementScope(scope)
	service.unconfigure()
	if (removeJKS == True):
		service.deleteRTSSKStore(scope)
	if (doSave == True):
		service.saveAll()
	#endif

#enddef

def configureServices(scope, sourceDir):

	service = RTSSAdminService()
	service.setManagementScope(scope)
	service.createRTSSKStore(scope)
	service.setPolicySetDirectory(sourceDir)
	service.configure()

	service.saveAll()

#enddef

def showServices(scope):

	service = RTSSAdminService()
	service.setManagementScope(scope)
	service.show()
	
#enddef

#
#  Pick a server in the target cluster, find it's node, and get the version of the node
def getClusterVersion(searchCluster):
	global True
	global TRACE
	
	## Find the cell id
	## wsadmin>$AdminConfig list Cell
	## c02rbt01Cell01(cells/c02rbt01Cell01|cell.xml#Cell_1)	
	cellId = AdminConfig.list("Cell")
	
	if (TRACE == True):
		print "cellId %s" %(cellId)
	#endif
	
	if (cellId == ""):
		message = "The cell name could not be retrieved."
		raise ConfigurationException(message)
	#endif
	
	###  List all of the server clusters
	## wsadmin>$AdminConfig list ServerCluster c02rbt01Cell01(cells/c02rbt01Cell01|cell.xml#Cell_1)
	## c02rbt01(cells/c02rbt01Cell01/clusters/c02rbt01|cluster.xml#ServerCluster_1260569674417)
	clusterIdList = AdminConfig.list("ServerCluster", cellId)
	if (TRACE == True):
		print "clusterIdList %s" %(clusterIdList)
	#endif

	if (clusterIdList == ""):
		message = "The cell %s does not have any clusters." %(cellId)
		raise ConfigurationException(message)
	#endif
	
	
	###  Look at the name of each cluster.. find the one we are looking for
	## wsadmin>$AdminConfig show c02rbt01(cells/c02rbt01Cell01/clusters/c02rbt01|cluster.xml#ServerCluster_1260569674417) name
	##{name c02rbt01}
	
	clusterName = ""
	clusterId  = ""
	if len(clusterIdList) > 0 :
			for clusterId in clusterIdList.split(lineSeparator):
				clusterName = AdminConfig.showAttribute(clusterId, "name")
				if (TRACE == True):
					print "clusterName %s" %(clusterName)
				#endif
				if (clusterName == searchCluster):
					print "Found target cluster %s" %(clusterName)
					break
				#endif
			#endfor
	#endif
	
	if (clusterName == ""):
		message = "The cluster %s could not be found in the cell." %(searchCluster, cellId)
		raise ConfigurationException(message)
	#endif
	
	###  look for the server members in the matched cluster id
	## wsadmin>$AdminConfig show c02rbt01(cells/c02rbt01Cell01/clusters/c02rbt01|cluster.xml#ServerCluster_1260569674417) members
	## {members {c02rbt02(cells/c02rbt01Cell01/clusters/c02rbt01|cluster.xml#ClusterMember_1260569685231)
	##   c02rbt03(cells/c02rbt01Cell01/clusters/c02rbt01|cluster.xml#ClusterMember_1260569688295)}}
	memberList  = AdminConfig.showAttribute(clusterId, "members")
	memberList = memberList.replace("[", "");
	serverIdList = memberList.replace("]", "");
	if (TRACE == True):
		print "serverIdList %s" %(serverIdList)
	#endif	
	
	if (serverIdList == ""):
		message = "The cluster %s does not have any members." %(searchCluster)
		raise ConfigurationException(message)
	#endif
	
	###  find the node name against the server id
	## just pick and use the first server in the cluster
	## wsadmin>$AdminConfig show c02rbt02(cells/c02rbt01Cell01/clusters/c02rbt01|cluster.xml#ClusterMember_1260569685231) nodeName
	## {nodeName c02rbt02Node01}

	nodeName = ""
	if (TRACE == True):
		print "Retrieving nodename"
	#endif	
	if len(serverIdList) > 0:
		for serverId in serverIdList.split(lineSeparator):
			if (TRACE == True):
				print "serverId %s" %(serverId)
			#endif
			nodeName = AdminConfig.showAttribute(serverId, "nodeName")
			if (len(nodeName) > 0):
				break
			#endif
	#endfor
	if (TRACE == True):
		print "nodeName %s" %(nodeName)
	#endif	
	
	if (nodeName == ""):
		messge = "A node name in the cluster %s could not be queried.  The cluster version can not be determined." %(searchCluster)
		raise ConfigurationException(message)
	#endif
		
	###  find the node version
	## wsadmin>$AdminTask getNodeBaseProductVersion  "-nodeName c02rbt02Node01"
	if ( len(nodeName) > 0 ):
		version = AdminTask.getNodeBaseProductVersion('[-nodeName '+nodeName+']')
	#endif
	
	if (TRACE == True):
		print "Websphere cluster version %s" %(version)
	#endif	
	
	return version

#enddef

def getNodeVersion(nodeName):
	global True
	global TRACE
	
	if (TRACE == True):
		print "Querying version for node %s" %(nodeName)
	#endif
	
	version = AdminTask.getNodeBaseProductVersion('[-nodeName '+nodeName+']')
	
	if (TRACE == True):
		print "Websphere node version %s" %(version)
	#endif
	
	return version
#enddef

def validate():
	global KEYSTOREPASSWORD
	global TRUSTSTOREPASSWORD
	global PASSWORDFILE
	global SOURCEDIR
	global PDT_NAME
	global ACTION
	global SUFFIX
	global CN
	global DN
	global KEYNAME
	global KEYALIAS
	global NODENAME
	global CLUSTERNAME
	global SCOPE
	global VERSION

	if ACTION == "show":
		return
	#endif

	if len(ACTION) == 0:
		print
		print "Action was not provided."
		print
		usage()
		sys.exit(-1)
	#endif

	if ACTION != "configure" and ACTION != "unconfigure" and ACTION != "reconfigure":
		print
		print "Specified action is not valid: %s" %(ACTION)
		print
		usage()
		sys.exit(-1)
	#endif
	
	if len(PDT_NAME) == 0:
		print
		print "RTSS PDT name was not provided."
		print
		usage()
		sys.exit(-1)
	#endif
	
	if len(KEYSTOREPASSWORD) == 0:
		if len(PASSWORDFILE) == 0:
			print
			print "Key store password was not provided; specify one of --kspassword or --passwordfile"
			print
			usage()
			sys.exit(-1)
		else:
			file = open(PASSWORDFILE, "rw")
			fpassword = file.read()
			# Strip leading/trailing whitespace, as some systems will append a newline to read-in pwd.
			password = fpassword.strip()
			file.close()
			if len(password) == 0:
				print
				print "No key store password was found in the password file " + PASSWORDFILE
				sys.exit(-1)
			else:
				KEYSTOREPASSWORD = password
				TRUSTSTOREPASSWORD = password
			#endif
		#endif
	else:
		if len(PASSWORDFILE) != 0:
			print
			print "Specify only one of --kspassword or --passwordfile"
			print
			usage()
			sys.exit(-1)
		#endif
	#endif
	
	if len(TRUSTSTOREPASSWORD) == 0:
		print
		print "Trust store password was not provided."
		print
		usage()
		sys.exit(-1)
	#endif	
	
	if ACTION != "unconfigure":
		if len(SOURCEDIR) == 0:
                        print
			print "Policy set source directory was not provided."
                        print
			usage()
			sys.exit(-1)
		#endif
                if os.path.isdir(SOURCEDIR) != True:
                        print
                        print "Cannot access the source directory."
                        print "Windows usage tip:  Use forward slashes and short names in directory paths.  For example, c:/PROGRA~1/IBM/RTSS/registration"
                        sys.exit(2)
                #endif
	#endif	
	

	SUFFIX = "OU=tivoli, O=ibm, C=us"
	CN = PDT_NAME
	DN = 'CN='+CN+', '+SUFFIX
	KEYNAME = DN
	KEYALIAS = PDT_NAME
	
		
	version = "7"
	
	# set the management scope
	if len(CLUSTERNAME) > 0 :
		#SCOPE = '(cell):'+CELLNAME+':(cluster):'+CLUSTERNAME
		SCOPE = '(cell):'+CELLNAME
		# figure out the WAS version
		#wsadmin>$AdminTask getNodeBaseProductVersion  "-nodeName c02rbt01CellManager01"
		#6.1.0.27
		
		version = getClusterVersion(CLUSTERNAME);

	else:
		# try to calculate the node name if not provided.
		if len(NODENAME) == 0:
			# node = AdminConfig.list("Node" )
			NODENAME = AdminConfig.showAttribute(AdminConfig.list("Node" ), "name" )
		#endif
		# figure out the WAS version
		version = getNodeVersion(NODENAME);
		
		SCOPE = '(cell):'+CELLNAME+':(node):'+NODENAME
	#endif
	

	print "WebSphere version = %s" %(version)
	
	VERSION = version


#enddef validate

def getopts():
	global True
	global TRACE
	global DEBUG
	global KEYSTOREPASSWORD
	global TRUSTSTOREPASSWORD
	global PASSWORDFILE
	global SOURCEDIR
	global PDT_NAME
	global ACTION
	global CLUSTERNAME
	
	action = ACTION 



	optString = "a:k:p:n:s:t:zh"
	longOpts = ["action=","kspassword=","passwordfile=","name=","sourcedir=","target=","zbug","help"]
	try:
		## debugging under RSA requires a different argv
		if (DEBUG == True):
			opts, args = getopt.getopt(sys.argv[1:], optString, longOpts)
		else:
			opts, args = getopt.getopt(sys.argv, optString, longOpts)
		#endif
		
	except getopt.GetoptError, err:
		# print help information and exit:
		print str(err)
		usage()
		sys.exit(2)
	#endtry
	
	#print opts
	for o, a in opts:
		#print "option = %s" %(o)
		if o in ("-h", "--help"):
			usage()
			sys.exit(0)
		elif o in ("-a", "--action"):
			action = a.lower()
		elif o in ("-t", "--target"):
			CLUSTERNAME = a
		elif o in ("-k", "--kspassword"):
			KEYSTOREPASSWORD = a
			TRUSTSTOREPASSWORD = a
		elif o in ("-p", "--passwordfile"):
			PASSWORDFILE = a
		elif o in ("-s", "--sourcedir"):
			SOURCEDIR = a
			#print "SOURCEDIR = " + SOURCEDIR
		elif o in ("-n", "--name"):
			PDT_NAME = a
		elif o in ("-z", "--zbug"):
			TRACE = True
		else:
			print "Unknown option %s" %(o)
			usage()
			sys.exit(-1)
		#endif
	#endfor

	optAction = action.lower();
	ACTION = optAction

	validate()

	return optAction
#end getopts



def usage():

	#basename = os.path.basename(sys.argv[0])
	print "Usage: --help | --action [configure | unconfigure | reconfigure | show] [--target cluster-name] --name pdt-name  { --kspassword rtss-jks-password | --passwordfile rtss-jks-password-file }  --sourcedir policyset-dir"
	print " Windows tip: Use forward slashes in directory path for sourcedir and passwordfile."
#end usage

def initialize():
	
	global APPLICATION
	global RTSS
	global RTSS_AGENT

	# figure out if RTSS or RTSS Agent is installed
	installedApp = ""
	installedAppsList = AdminApp.list()
	installedApps = installedAppsList.split('\n')
	for app in installedApps:
		if app.startswith(RTSS_AGENT):
			installedApp = RTSS_AGENT
			break
		elif  app.startswith(RTSS):
			installedApp = RTSS
			break
		#endif
	#endfor
	
	if len(installedApp) == 0 :
		# Neither RTSS or RTSS client are installed
		print 'Either "%s" or "%s" needs to be installed prior to invoking this utility.' %(RTSS, RTSS_AGENT)
		sys.exit(2)
	#endif
	
	APPLICATION = installedApp
	print "Configuring WS-Security for %s" %(APPLICATION)
#enddef

def main():
	
	try:
		# set global defaults
		initialize()
		
		# Parse command line arguments
		optAction = getopts()
		print "Performing action %s" %(optAction)
		
		# Perform operation
		print "Starting configuration operation %s" %(optAction)
		
		if optAction == "default":
			showServices(SCOPE)
		elif optAction == "show":
			showServices(SCOPE)
		elif optAction == "configure":
			configureServices(SCOPE, SOURCEDIR)
		elif optAction == "unconfigure":
			# removeJKS = False = do not remove the RTSS keystore configuration in WAS (def 97695)
			# save = True = save after running unconfig
			unconfigureServices(SCOPE, False, True)
		elif optAction == "reconfigure":
			# removeJKS = False = do not remove the RTSS keystore configuration in WAS
			# save = False = do not save after unconfig (save after config instead)
			unconfigureServices(SCOPE, False, False)
			configureServices(SCOPE, SOURCEDIR)
		#endif
		
		print
		print "Configuration operation completed."

		try:
			if len(PASSWORDFILE) > 0:
				print
				print "Removing password file " + PASSWORDFILE
				print
				os.remove(PASSWORDFILE)
			#endif
		except OSError, err:
			print
			print str(err)
			print "Error removing password file " + PASSWORDFILE + ".  Remove this file manually."
		#endtry
	except:
		print
		print "Configuration encountered errors."
		e = sys.exc_info()[1]
		print e
	#endtry
#end main



main()

