/* %Z%%M% %I% %W% %G% %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * 5724-S24
 * (C) Copyright IBM Corp. 2010
 * ===========================================================================
 */
package com.ibm.tscc.rtss.authz.sample.erule;

import java.util.List;
import java.util.Properties;

import com.ibm.tscc.rtss.authz.api.Attribute;
import com.ibm.tscc.rtss.authz.api.IExternalRule;
import com.ibm.tscc.rtss.authz.api.RequestContext;

/**
 * External rule sample.  
 * 
 * <p>
 * An external rule can be used to call logic for a Permit or Deny outcome. 
 * The outcome returned from the external rule may be combined with standard rule parameter conditions 
 * to address more complex scenarios. 
 * 
 * <p>
 * The following configuration is required in the security-services.xmi file, in the {@code <components name="Authz">} section:
 * 
 * <pre>
 * {@code
 *    <subComponents name="ExternalRules">
 *       <items name="server1-rules-engine">
 *         <properties>
 *           <values name="id" value="com.ibm.tscc.rtss.authz.sample.erule.RiskClassificationRuleSample" type="java.lang.String" />
 *           <values name="enabled" value="true" type="java.lang.String" />
 *           <values name="Hostname" value="server1.customer.com" type="java.lang.String" />
 *           <values name="Username" value="foo" type="java.lang.String" />
 *         </properties>
 *       </items>
 *       <items name="server2-rules-engine">
 *         <properties>
 *           <values name="id" value="com.ibm.tscc.rtss.authz.sample.erule.RiskClassificationRuleSample" type="java.lang.String" />
 *           <values name="enabled" value="true" type="java.lang.String" />
 *           <values name="Hostname" value="server2.customer.com" type="java.lang.String" />
 *           <values name="Username" value="bar" type="java.lang.String" />
 *         </properties>
 *       </items>
 *    </subComponents>
 *  }
 *  </pre>
 *    
 */
public class RiskClassificationRule implements IExternalRule
{
    public final static String RiskClassificationRuler_java_sourceCodeID = "$Id: %Z%%M%    %I%  %W%  %G%  %U% $";
    public static final String RiskClassificationRule_java_copyright = "Licensed Materials - Property of IBM - All Rights Reserved. 5724-S24  (C) Copyright IBM Corp. 2010";
    
    private Properties _properties;

    public Decision evaluate( RequestContext context ) {
        
        List<Attribute> subjectId = context.getSubjectAttributes("urn:oasis:names:tc:xacml:1.0:subject:subject-id", 
                "http://www.w3.org/2001/XMLSchema#string", null );
        
        System.out.println( "RiskClassificaitionRule#evaluate()" );
        System.out.println( "subject-id: " + subjectId );
        
        if (_properties != null)
        {
            System.out.println( "Configuration Properties: " );
            _properties.list(System.out);
        }
        
        return Decision.PERMIT;
    }

    public void shutdown() {
        
        System.out.println( "shutdown()" );

    }

    public void startup( Properties props ) {
        
        System.out.println( "startup()" );
        _properties = props;
    }


}
