/* %Z%%M% %I% %W% %G% %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * 5724-S24
 * (C) Copyright IBM Corp. 2010
 * ===========================================================================
 */
package com.ibm.tscc.rtss.authz.sample;

import java.io.IOException;
import java.io.Writer;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.security.auth.Subject;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.ws.BindingProvider;

import oasis.names.tc.xacml._2_0.context.schema.os.ActionType;
import oasis.names.tc.xacml._2_0.context.schema.os.AttributeType;
import oasis.names.tc.xacml._2_0.context.schema.os.AttributeValueType;
import oasis.names.tc.xacml._2_0.context.schema.os.DecisionType;
import oasis.names.tc.xacml._2_0.context.schema.os.EnvironmentType;
import oasis.names.tc.xacml._2_0.context.schema.os.RequestType;
import oasis.names.tc.xacml._2_0.context.schema.os.ResourceType;
import oasis.names.tc.xacml._2_0.context.schema.os.ResponseType;
import oasis.names.tc.xacml._2_0.context.schema.os.ResultType;
import oasis.names.tc.xacml._2_0.context.schema.os.StatusCodeType;
import oasis.names.tc.xacml._2_0.context.schema.os.StatusType;
import oasis.names.tc.xacml._2_0.context.schema.os.SubjectType;
import oasis.names.tc.xacml._2_0.policy.schema.os.AttributeDesignatorType;

import com.ibm.tivoli.security.schemas.entitlements.AttributeDesignatorsType;
import com.ibm.tivoli.security.schemas.entitlements.EntitlementRequestType;
import com.ibm.tivoli.security.schemas.entitlements.EntitlementResponseType;
import com.ibm.tivoli.security.schemas.entitlements.EntitlementType;
import com.ibm.tscc.rtss.authz.ws.xacml.rtss.authzservice.AuthzService;
import com.ibm.tscc.rtss.authz.ws.xacml.rtss.authzservice.AuthzService_Service;
import com.ibm.websphere.security.auth.WSSubject;
import com.ibm.websphere.security.cred.WSCredential;
import com.ibm.websphere.wsaddressing.EndpointReferenceCreationException;
import com.ibm.websphere.wsaddressing.WSAConstants;


/**
 * Servlet implementation class AuthzSampleServlet
 */
public class AuthzSampleServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
	private static final String STRING_TYPE = "http://www.w3.org/2001/XMLSchema#string";
	private static final String X500_TYPE = "urn:oasis:names:tc:xacml:1.0:data-type:x500Name";
	private static final String ENTITLEMENT_STATUS_OK = "urn:oasis:names:tc:xacml:1.0:status:ok";
    /**
     * @see HttpServlet#HttpServlet()
     */
    public AuthzSampleServlet() {
        super();
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// TODO Auto-generated method stub
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//first validate that the request comes from the UI
	    String hostname = request.getParameter("host");
	    if (hostname == null || hostname.length() == 0){
	        hostname = "localhost";
	    }
	    String port = request.getParameter("port");
	    if (port == null || port.length() == 0){
	        port = "9080";
	    }
	    String rtssuser = request.getParameter("rtssadmin");
	    String rtsspassword = request.getParameter("rtsspassword");
	    //TODO validate user and password
	    
	    String testType = request.getParameter("testType");
	    if (testType == null || testType.length() == 0){
	        testType = "authorization";
	    }
	    String appid = request.getParameter("context");
	    AuthzService service = null;
	    try{
	        service = getWebServiceClient(rtssuser, rtsspassword, hostname, port);
	    }catch(Exception e){
	        throw new ServletException(e);
	    }
	    String result = "";
	    if (testType.equals("authorization")){
	        String action = request.getParameter("action");
	        String resource = request.getParameter("resource");
	        result = testAuthorization(service, action, resource, appid);
	    }else{
	        String decissionType = request.getParameter("decissionType");
	        result = testEntitlements(service, appid, decissionType);
	    }
	    Writer out = response.getWriter();
	    out.write("<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"images/style.css\" media=\"screen\" />\n" +
	    		"</head><body>\n");
	    out.write("<div id=\"wrap\"><div id=\"header\"><h1>Response from this test is:</h1>\n</div><br>");
	    out.write(result);
	    out.write("<br><hr>");
	    String redirectUrl = request.getParameter("redirectUrl");
	    if (redirectUrl  == null || redirectUrl.length() == 0){
	        redirectUrl = "index.html";
	    }
	    out.write("<a href=\"" + redirectUrl + "\"><b><--back</b></a>");
	    out.write("</body></html>");
	}

	
	private AuthzService getWebServiceClient(String rtssuser, String rtsspassword, String host, String port) throws URISyntaxException, EndpointReferenceCreationException{
	    AuthzService_Service service = new AuthzService_Service();
	    AuthzService soap = service.getAuthzServiceSOAP();
	    String serviceURL = "http://" + host + ":" + port + "/rtss/authz/services/AuthzService";
	    com.ibm.wsspi.wsaddressing.EndpointReference epr = null;
	    
	    BindingProvider bp = (BindingProvider)soap;
	    Map<String, Object> reqAttrs = bp.getRequestContext();
	    URI consumerEndpoint = new URI(serviceURL);
        epr = com.ibm.wsspi.wsaddressing.EndpointReferenceManager.createEndpointReference(consumerEndpoint);
        
        reqAttrs.put(WSAConstants.WSADDRESSING_DESTINATION_EPR, epr);
        reqAttrs.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        
	    reqAttrs.put(javax.xml.ws.BindingProvider.USERNAME_PROPERTY, rtssuser);
        reqAttrs.put(javax.xml.ws.BindingProvider.PASSWORD_PROPERTY, rtsspassword);
	    return soap;
	}
	
	private String testAuthorization(AuthzService service, String actionStr, String resourceStr, String appid) throws IOException{
	    String result = DecisionType.DENY.toString();
	    try
        {
	        //To build and authorization request we need the following
	        // 1) An access subject (who is accessing)
	        // 2) An optional requesting machine subject (describes the entity
	        //    making the request, usually the PEP).
	        // 3) A resource (what is the access subject requesting access to)
	        // 4) An action (what is the access subject doing on the resource)
	        // 5) An environment context (denotes an evaluation context, usually
	        //    the id of the application, WebService, etc)
            SubjectType asubject = buildAccessSubject();
            SubjectType machineSubject = buildRequestingMachine();
            ActionType action = buildAction(actionStr);
            ResourceType resource = buildResource(resourceStr);
            EnvironmentType context = buildContext(appid); 
            
            RequestType request = new RequestType();
            request.getSubject().add(asubject);
            request.getSubject().add(machineSubject);
            request.getResource().add(resource);
            request.setAction(action);
            request.setEnvironment(context);
            
            ResponseType response = service.evaluateXACML(request);
            if (response.getResult() != null && response.getResult().size() > 0){
                ResultType resulttype = response.getResult().get(0);
                result = resulttype.getDecision().toString();
            }
        } catch (Exception e)
        {
            result = "FAILED TO EVALUATE REQUEST: " + e.getMessage();
            throw new IOException(e);
        }
	    return result;
	}
	
	private String testEntitlements(AuthzService service, String appid, String dectyp) throws IOException{
	    //An entitlement request consists of the entitlementrequest type wrapper, we then
	    //pass what we know (like in an authz request)
	    // 1) The subject that is trying to access a resource
	    // 2) An environment context (denotes an evaluation context, usually
        //    the id of the application, WebService, etc)
	    // 3) No action or resources are passed into the entitlement request, 
	    //    instead we use attribute designators that define what we want to
	    //    get back, in this example we want to get entitlements for the
	    //    resource/action.
	    // 4) Decision, limits the decision we want to get entitlements for, if
	    //    the decision is "permit" we get the entitlements for the resource-action
	    //    that the user can access.
	    String result = "";
	    EntitlementRequestType request = new EntitlementRequestType();
	    RequestType xacmlrequest = new RequestType();
	    try{
	        //use our authenticated subject
	        SubjectType asubject = buildAccessSubject();
	        xacmlrequest.getSubject().add(asubject);
	        //build the environment object
	        EnvironmentType context = buildContext(appid); 
	        xacmlrequest.setEnvironment(context);
	        request.setRequest(xacmlrequest);
	        //attribute designators are fixed
	        request.setAttributeDesignators(getAttributeDesignators());
	        request.setDecision(DecisionType.fromValue(dectyp));
	        
	        //send the request to the RTSS server
	        EntitlementResponseType response = service.evaluateEntitlements(request);
	        StatusType status = response.getStatus();
	        StatusCodeType st = status.getStatusCode();
	        if (st.getValue().equals(ENTITLEMENT_STATUS_OK)){
                result = "<h3>Entitlements for Decission: " + dectyp + "</h3><br>";
                result += "<table class=\"stats\" id='test_table' style='border: 1px blue groove;'>";
                result +="\n<tr>\n<td class=\"hed\">Subject</td><td class=\"hed\">Resource</td><td class=\"hed\">Action</td></tr>";
	            //we got entitlements back, lets walk them
	            List<EntitlementType> entitlements = response.getEntitlement();
	            if (entitlements == null){
	                result += "</table><br>";
	                result += "<h2>No entitlements found</h2>";
	                entitlements = Collections.emptyList();
	            }
	            for (EntitlementType entitlement  : entitlements){
	               String subject = "";
	               String resource = "";
	               String action = "";
	               SubjectType sbj = entitlement.getSubject();
	               if (sbj != null && sbj.getAttribute() != null && sbj.getAttribute().size() > 0){
	                   List<AttributeType> sbjattribs = sbj.getAttribute();
	                   for (AttributeType sattrib : sbjattribs){
	                       List<AttributeValueType>values = sattrib.getAttributeValue();
	                       for (AttributeValueType value : values){
	                           subject += value.getContent().toString() + " ";
	                       }
	                   }
	               }
	               ResourceType res = entitlement.getResource();
	               if (res != null && res.getAttribute() != null && res.getAttribute().size() > 0){
	                   List<AttributeType> resAttribs = res.getAttribute();
	                   for (AttributeType sattrib : resAttribs){
                           List<AttributeValueType>values = sattrib.getAttributeValue();
                           for (AttributeValueType value : values){
                               resource += value.getContent().toString() + " ";
                           }
                       }
	               }
	               ActionType actn =  entitlement.getAction();
	               if (actn != null && actn.getAttribute() != null && actn.getAttribute().size() > 0){
                       List<AttributeType> attribs = actn.getAttribute();
                       for (AttributeType sattrib : attribs){
                           List<AttributeValueType>values = sattrib.getAttributeValue();
                           for (AttributeValueType value : values){
                               action += value.getContent().toString() + " ";
                           }
                       }
                   }
	               result += "\n<tr><td>" + subject + "</td><td>" + resource + "</td><td>" + action + "</td></tr>";
	            }
	            if (entitlements.size() > 0){
	                result += "</table>";
	            }
	        }else{
	            result = st.getValue();
	        }
	    }catch(Exception e){
	        result = "FAILED TO EVALUATE REQUEST: " + e.getMessage();
            throw new IOException(e);
	    }
	    return result;
	}
	
	private SubjectType buildAccessSubject() throws Exception{
	    //first set the subject type which is "access-subject"
	    SubjectType subject = new SubjectType();
	    subject.setSubjectCategory("urn:oasis:names:tc:xacml:1.0:subject-category:access-subject");
	    //calculate the calling subject
	    String userId = WSSubject.getCallerPrincipal();
	    //add the subject id to the subject (who is the principal making the call)
	    subject.getAttribute().add(
	            buildAttribute("urn:oasis:names:tc:xacml:1.0:subject:subject-id", 
	                           STRING_TYPE, 
	                           userId));
	    //now calculate the groups and add them to the subject    
	    Subject authenticatedSubj = WSSubject.getCallerSubject();
	    
	    if (authenticatedSubj == null){
	        throw new Exception("Unable to retrieve calling subject");
	    }

	    
	    Set<WSCredential> credSet= authenticatedSubj.getPublicCredentials(WSCredential.class);
	    if (credSet == null || credSet.size() != 1){
	        throw new Exception("Unable to calculate credentials");
	    }
	    WSCredential cred = credSet.iterator().next();
	    List groups = cred.getGroupIds();
	    //when WebSphere returns the groups via the WSgroup:defaultWIMFileBasedRealm/
        for (Iterator it = groups.iterator(); it.hasNext(); ){
	        String groupstr = (String)it.next();
	        groupstr = groupstr.substring(groupstr.indexOf("/") + 1);
	        subject.getAttribute().add(
	                buildAttribute(
	                        "urn:oasis:names:tc:xacml:1.0:subject:group-id",
	                        STRING_TYPE,
	                        groupstr));
	        //for groups we can put them in the request as a string or as an
	        //x500
	        subject.getAttribute().add(
                    buildAttribute(
                            "urn:oasis:names:tc:xacml:1.0:subject:group-id",
                            X500_TYPE,
                            groupstr));
	    }
	    
	    //We have populated the subject with the user id and the group
	    //membership, additionally authenitcation tokens could be 
	    //added here. 
//	    byte[] token = cred.getCredentialToken();
//	    if (token != null && token.length > 0){
//	        subject.getAttribute().add(
//	                //the attribute id needs to match this value for RTSS to know it has a token
//	                buildAttribute("http://security.tivoli.ibm.com/attributes/subject/authenticationToken"
//	                        , STRING_TYPE, 
//	                        new String(token)));
//	    }
	    return subject;
	}
	
	private SubjectType buildRequestingMachine(){
	    SubjectType subject = new SubjectType();
        subject.setSubjectCategory("urn:oasis:names:tc:xacml:1.0:subject-category:requesting-machine");
        //set this "sample" as the id
        subject.getAttribute().add(buildAttribute("urn:oasis:names:tc:xacml:1.0:subject:subject-id", 
                                                   STRING_TYPE, "AUTHZ-SAMPLE"));
        //fill some rtss audit attributes to get the audit trail
        subject.getAttribute().add(buildAttribute("http://rtss.tscc.ibm.com/audit/application", 
                STRING_TYPE, "AUTHZ-SAMPLE"));
        subject.getAttribute().add(buildAttribute("http://rtss.tscc.ibm.com/audit/component", 
                STRING_TYPE, "AUTHZ-SAMPLE-PEP"));
        subject.getAttribute().add(buildAttribute("http://rtss.tscc.ibm.com/audit/componentIdType", 
                STRING_TYPE, "AUTHZ-SAMPLE-PEP"));
        subject.getAttribute().add(buildAttribute("http://rtss.tscc.ibm.com/audit/instanceId", 
                STRING_TYPE, "AUTHZ-SAMPLE"));
        subject.getAttribute().add(buildAttribute("http://rtss.tscc.ibm.com/audit/executionEnvironment", 
                STRING_TYPE, "AUTHZ-SAMPLE"));
        subject.getAttribute().add(buildAttribute("http://rtss.tscc.ibm.com/audit/componentType", 
                STRING_TYPE, "AUTHZ-SAMPLE-PEP"));
        return subject;
	}
	
	
	private ActionType buildAction(String actionStr){
	    ActionType action = new ActionType();
	    action.getAttribute().add(
	            buildAttribute("urn:oasis:names:tc:xacml:1.0:action:action-id", 
	                    STRING_TYPE, 
	                    actionStr));
	    return action;
	}
	
	private ResourceType buildResource(String resource){
	    ResourceType res = new ResourceType();
	    res.getAttribute().add(buildAttribute("urn:oasis:names:tc:xacml:1.0:resource:resource-id",
	                                          STRING_TYPE,
	                                          resource));
	    return res;
	}
	
	private EnvironmentType buildContext(String context){
	    EnvironmentType env = new EnvironmentType();
	    AttributeType  attrib = buildAttribute("ContextId", STRING_TYPE, context);
	    attrib.setIssuer("http://security.tivoli.ibm.com/policy/distribution");
	    env.getAttribute().add(attrib);
	    return env;
	}
	
	private AttributeDesignatorsType getAttributeDesignators(){
	    AttributeDesignatorsType designators = new AttributeDesignatorsType();
	    //designator for action
	    AttributeDesignatorType actiondesignator = new AttributeDesignatorType();
	    actiondesignator.setAttributeId("urn:oasis:names:tc:xacml:1.0:action:action-id");
	    actiondesignator.setDataType(STRING_TYPE);
	    designators.getActionAttributeDesignator().add(actiondesignator);
	    //designator for resource
	    AttributeDesignatorType resourcedesignator = new AttributeDesignatorType();
	    resourcedesignator.setAttributeId("urn:oasis:names:tc:xacml:1.0:resource:resource-id");
	    resourcedesignator.setDataType(STRING_TYPE);
	    designators.getResourceAttributeDesignator().add(resourcedesignator);
	    return designators;
	}
	
	private AttributeType buildAttribute(String id, String type, Object value){
	    AttributeType attrib = new AttributeType();
	    attrib.setAttributeId(id);
	    attrib.setDataType(type);
	    //the lists that we get from the JAXB objects are mutable and
	    //manipulate the object content directly
	    List<AttributeValueType> values = attrib.getAttributeValue();
	    AttributeValueType avt = new AttributeValueType();
	    avt.getContent().add(value);
	    values.add(avt);
	    return attrib;
	}
	
	private AttributeType buildAttribute(String id, String type, String[] values){
        AttributeType attrib = new AttributeType();
        attrib.setAttributeId(id);
        attrib.setDataType(type);
        //the lists that we get from the JAXB objects are mutable and
        //manipulate the object content directly
        List<AttributeValueType> valueslist = attrib.getAttributeValue();
        for (String value : values){
            AttributeValueType avt = new AttributeValueType();
            avt.getContent().add(value);
            valueslist.add(avt);
        }
        return attrib;
    }
}
