/* %Z%%M% %I% %W% %G% %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * 5724-S24
 * (C) Copyright IBM Corp. 2010
 * ===========================================================================
 */
package com.ibm.tscc.rtss.jaccplus.sample.authz;

import java.io.IOException;
import java.io.PrintWriter;
import java.security.Permission;
import java.security.PermissionCollection;
import java.util.Enumeration;

import javax.security.jacc.PolicyContextException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.sec.authz.jaccplus.ApplicationPermission;
import com.ibm.sec.authz.jaccplus.ApplicationPolicy;
import com.ibm.sec.authz.jaccplus.ContainerEvaluationContext;
import com.ibm.sec.authz.jaccplus.EvaluationContext;

/**
 * <p>
 * Servlet implementation class GetEntitlementsServlet.  This sample shows how to use the 
 * JACCPLUS API for entitlements requests.  
 *
 * <p> For this sample, create a custom application service in Tivoli Security Policy Manager (TSPM):
 * <pre>
 *    MyAppService
 *       Element1
 *          Element11
 *       Element2 
 *          Element22
 * </pre>
 * 
 * <p>
 * Default values used for entitlements request:
 *    <ul>
 *       <li>Context ID: MyAppServiceID</li>
 *    </ul>
 *    
 * <p>
 * Create TSPM policy for the service with various resources and actions mapped to groups and distribute to Tivoli Runtime Security Services (RTSS).
 * This application should be installed where the RTSS client is installed.
 *
 *<p>
 * To execute sample, bring up browser with url: <br>
 * http://localhost:9080/rtss/sample/jaccplus/entitlements.jsp
 * 
 */
public class GetEntitlementsServlet extends HttpServlet 
{
    public static final String GetEntitlementsServlet_java_sourceCodeID = "$Id: %Z%%M%    %I%  %W%  %G%  %U% $";
    public static final String GetEntitlementsServlet_java_copyright = "Licensed Materials - Property of IBM - All Rights Reserved. 5724-S24  (C) Copyright IBM Corp. 2010";

	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public GetEntitlementsServlet() {
        super();
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
    {
        doIt(request, response);
    }   
    
    /* (non-Java-doc)
     * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
    {
        doIt(request, response);
    } 
    
    private void doIt(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        PrintWriter out;
        String title = "Entitlements Servlet Output";

        // set content type and other response header fields first
        response.setStatus(HttpServletResponse.SC_OK);
        response.setContentType("text/html");

        // then write the data of the response
        out = response.getWriter();

        out.println("<HTML><HEAD><TITLE>");
        out.println(title);
        out.println("</TITLE></HEAD><BODY>");
        out.println("<H1>" + title + "</H1>");
        out.println("<P>This is the result from authorization : ");
        
        doBody(out, request, response);
        
        out.println("</BODY></HTML>");
        out.close();
        
    }

    private void doBody(PrintWriter out, HttpServletRequest request, HttpServletResponse response)
    {
        String contextId = request.getParameter("contextID");

        if ((contextId == null) || contextId.length() == 0)
        {
           contextId = "MyAppServiceID";
        }
        // Call the getPermissions() API. Please note that only
        // ApplicationPermission and GenericRoleRefPermission
        // are supported .
        PermissionCollection entitledPerms = null;
        try
        {
            // Use either implementation of EvaluationContext
            EvaluationContext context = new ContainerEvaluationContext();
            entitledPerms = ApplicationPolicy.getPolicy().getPermissions(contextId, context, ApplicationPermission.class);
        } catch (PolicyContextException e1)
        {

        }

        if (entitledPerms != null)
        {
            out.println("<ol>");
            // Loop over the returned Permissions 
            Enumeration<Permission> e = entitledPerms.elements();
            while (e.hasMoreElements())
            {
                out.print("<li>");
                Permission thisPerm = e.nextElement();
                out.print(thisPerm.getName());
                out.print(":");
                out.println(thisPerm.getActions());
                out.print("</li>");
            }
            out.println("</ol>");

        }

    }

}
